<?php
use kartik\datecontrol\Module;

$params = array_merge(
    require(__DIR__ . '/../../common/config/params.php'),
    require(__DIR__ . '/../../common/config/params-local.php'),
    require(__DIR__ . '/params.php'),
    require(__DIR__ . '/params-local.php')
);

return [
    'id' => 'app-backend',
    'basePath' => dirname(__DIR__),
	'language' => 'ru',
    'controllerNamespace' => 'backend\controllers',
    'bootstrap' => ['log'],
    'components' => [
		'formatter' => [
			'dateFormat' => 'yyyy.MM.dd',
			'timeFormat' => 'hh:i:ss',
			'datetimeFormat' => 'yyyy.MM.dd H:i:s',
			'decimalSeparator' => ',',
			'thousandSeparator' => ' ',
			'currencyCode' => 'EUR',
		],
        'user' => [
            'identityClass' => 'common\models\User',
			'enableAutoLogin' => false,
        ],
        'log' => [
            'traceLevel' => YII_DEBUG ? 3 : 0,
            'targets' => [
                [
                    'class' => 'yii\log\FileTarget',
                    'levels' => ['error', 'warning'],
                ],
            ],
        ],
        'errorHandler' => [
            'errorAction' => 'site/error',
        ],
		'urlManager' => [
			'enablePrettyUrl' => true,
			'enableStrictParsing' => false,
			'showScriptName' => false,
			'rules' => [
				'login' => 'site/login',
				'logout' => 'site/logout',
			],
		],

		'menu' => [
			'class' => 'common\components\RbacMenu',
			'items' => [
				[
					'label' => 'Документы',
					'url' => ['/document/index'],
//					'items'	=> [
//						['label' => 'Example', 'url' => ['/']],
//						[
//							'label' => 'Example', 'url' => ['/'],
//							'items'	=> [
//								['label' => 'Example', 'url' => ['/']],
//							]
//						],
//					]
				],
				[
					'label'	=> 'Ошибочные',
					'url'	=> ['/document/invalid']
				],
				[
					'label'	=> 'Настройки',
					'url'	=> ['/terminal/index']
				]
			],
		],
	],
	'modules' => [
		// http://demos.krajee.com/field-range
		'datecontrol' => [
			'class' => 'kartik\datecontrol\Module',

			// format settings for displaying each date attribute
			'displaySettings' => [
				Module::FORMAT_DATE => 'Y-m-d',
				Module::FORMAT_TIME => 'H:i:s A',
				Module::FORMAT_DATETIME => 'Y-m-d H:i:s A',
			],

			// format settings for saving each date attribute
			'saveSettings' => [
				Module::FORMAT_DATE => 'U', // saves as unix timestamp
				Module::FORMAT_TIME => 'H:i:s',
				Module::FORMAT_DATETIME => 'Y-m-d H:i:s',
			],

			// automatically use kartik\widgets for each of the above formats
			'autoWidget' => true,

			// use ajax conversion for processing dates from display format to save format.
			'ajaxConversion' => false,

			// default settings for each widget from kartik\widgets used when autoWidget is true
			'autoWidgetSettings' => [
				Module::FORMAT_DATE => ['type' => 2, 'pluginOptions' => ['autoclose' => true]], // example
				Module::FORMAT_DATETIME => [], // setup if needed
				Module::FORMAT_TIME => [], // setup if needed
			],

			// custom widget settings that will be used to render the date input instead of kartik\widgets,
			// this will be used when autoWidget is set to false at module or widget level.
			'widgetSettings' => [
				Module::FORMAT_DATE => [
					'class' => 'yii\jui\DatePicker', // example
					'options' => [
						'options' => ['class' => 'form-control'],
						'clientOptions' => ['dateFormat' => 'dd-mm-yy'],
					]
				]
			]
		]
	],
    'params' => $params,
];
