<?php

namespace backend\controllers;

use common\base\Controller;
use common\models\Document;
use common\models\search\Document as DocumentSearch;
use Yii;
use yii\base\Exception;
use yii\data\ActiveDataProvider;
use yii\helpers\Url;
use yii\web\NotFoundHttpException;

/**
 * DocumentController implements the CRUD actions for Document model.
 */
class DocumentController extends Controller
{
    /**
     * Lists all Document models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new DocumentSearch();

		$dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
			'searchModel' => $searchModel,
			'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Document model.
     * @param string $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

	
	public function actionInvalid() 
	{
		if ($selectedIds = Yii::$app->request->post('selection')) {
			$docManager = Yii::$app->getModule('docManager');
			
			$list = Document::find()->where(['id' => $selectedIds])->all();
			
			$count = 0;
			foreach($list as $document) {
				try {
					$docManager->documentFixSender($document);
					$docManager->documentSign($document);
					$docManager->documentSend($document);
					
					$count++;
				} catch (Exception $ex) {
					Yii::$app->session->setFlash('error', Yii::t('app', 'Error in document #{id}: {message} ', [
						'id'		=> $document->id,
						'message'	=> $ex->getMessage(),
					]));
				}
			}
			
			Yii::$app->session->setFlash('info', Yii::t('app', 'Fixed {count} documents', ['count'	=> $count]));
			
			return $this->refresh();
		}
		
		$dataProvider = new ActiveDataProvider([
			'query'	=> Document::find()->where([
							'statusInner'	=> Document::STATUS_INNER_SENDER_INVALID
						])
		]);
				
		
		return $this->render('invalid', [
			'dataProvider'	=> $dataProvider
		]);
	}

	public function actionUpdate($id, $action) {
		
		$docManager = Yii::$app->getModule('docManager');

		$document = Document::findOne(['id' => $id]);
		if (empty($document)) {
			throw new NotFoundHttpException("Can't find entity");
		}
		
		if ('fixSender' == $action) {
			try {
			
				$docManager->documentFixSender($document);
			} catch (Exception $e) {
				
			}
			
			if ($document->hasErrors()) {
				Yii::$app->session->setFlash('error', Yii::t('app', 'Document cannot be fixed'));
			} else {
				Yii::$app->session->setFlash('info', Yii::t('app', 'Document fixed'));
			}

			return $this->redirect(Url::toRoute(['document/view', 'id' => $document->id]));
			
		} elseif ('sign' == $action) {
			$error = null;
			
			try {
				if (!$docManager->documentSign($document)) {
					$error = 'Document sign failed';
				}
			} catch (Exception $ex) {
				$error = $ex->getMessage();
			}
			
			if ($error) {
				
				Yii::$app->session->setFlash('error', Yii::t('app', $error));
				
			} else {
				
				Yii::$app->session->setFlash('info', Yii::t('app', 'Document is signed'));
				
			}
			
			return $this->redirect(Url::toRoute(['document/view', 'id' => $document->id]));
		
			
		} elseif ('send' == $action) {
			
			
			if (!$docManager->documentSend($document)) {
				Yii::$app->session->setFlash('error', Yii::t('app', 'Документ не отправлен'));
			}
			
			Yii::$app->session->setFlash('info', Yii::t('app', 'Документ отправлен'));
			
			return $this->redirect(Url::toRoute(['document/view', 'id' => $document->id]));
			
		} else {
			throw new NotFoundHttpException("Unknown action '{$action}'");
		}
	}
	
    /**
     * Finds the Document model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param string $id
     * @return Document the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Document::findOne($id)) !== null) {
            return $model;
        } else {
			throw new NotFoundHttpException();
        }
    }
}
