<?php

namespace backend\controllers;

use Yii;
use common\base\Controller;
use common\models\form\Login;
use yii\web\ErrorAction;
use yii\web\HttpException;

/**
 * Site controller
 */
class SiteController extends Controller {

	public function actionIndex() {
		//return $this->redirect(Yii::$app->urlManager->createUrl('site/login'));
		return $this->render('index');
	}

	public function actionLogin() {
		if (!\Yii::$app->user->isGuest) {
			return $this->goHome();
		}
		
		$model = new Login();
		if ($model->load(Yii::$app->request->post()) && $model->login()) {
			return $this->render('hello', [
				'model' => $model,
			]);
		} else {
			return $this->render('login', [
				'model' => $model,
			]);
		}
	}

	public function actionLogout() {
		Yii::$app->user->logout();
		return $this->goHome();
	}


	public function actionError()
	{
		$exception = Yii::$app->errorHandler->exception;
		if ($exception instanceof HttpException) {
			$code = $exception->statusCode;
		} else {
			$code = $exception->getCode();
		}

		/**
		 * Перехватываем обработку базовых HTTP ошибок, дабы стандартизировать сообщения отдаваемые пользователям
		 * т.к. на данный момент не понятно как быть с переводами сообщений отдаваемых черех HttpException (иногда они не содержат конструкций подразумевающих перевод)
		 * жестко задаем содержимое сообщений в данном методе
		 */
		if ($code === 400) {
			return $this->render('error', [
				'exception' => $exception,
				'name' => Yii::t('app/error', 'Bad Request'),
				'message' => Yii::t('app/error', 'The request cannot be fulfilled due to bad syntax.'),
			]);
		} elseif ($code === 401) {
			return $this->render('error', [
				'exception' => $exception,
				'name' => Yii::t('app/error', 'Unauthorized'),
				'message' => Yii::t('app/error', 'Authentication is required.'),
			]);
		} elseif ($code === 403) {
			return $this->render('error', [
				'exception' => $exception,
				'name' => Yii::t('app/error', 'Forbidden'),
				'message' => Yii::t('app/error', 'The request was a valid request, but the server you have not access for this page.'),
			]);
		} elseif ($code === 404) {
			return $this->render('error', [
				'exception' => $exception,
				'name' => Yii::t('app/error', 'Not Found'),
				'message' => Yii::t('app/error', 'The requested page could not be found.'),
			]);
		} elseif ($code === 500) {
			return $this->render('error', [
				'exception' => $exception,
				'name' => Yii::t('app/error', 'Internal Server Error'),
				'message' => Yii::t('app/error', 'Internal server error happen'),
			]);
		}

		// используем стандартный механизм
		return (new ErrorAction($this->id, $this, [
			'view' => 'error'
		]))->run();
	}

}
