<?php
namespace common\components;

use Yii;
use yii\base\Component;

class RbacMenu extends Component implements \ArrayAccess
{

	/**
	 * Весь набор меню
	 * @var array
	 */
	protected $_items;
	/**
	 * Набор разрешенных элементов
	 * @var array
	 */
	protected $_permitted;

	/**
	 * @param $items
	 */
	public function setItems($items)
	{
		$this->_items = $items;
	}

	/**
	 * Получение всех разрешенных значений
	 * @return array
	 */
	public function getItems()
	{
		if (isset($this->_permitted)) {
			return $this->_permitted;
		}

		$auth = \Yii::$app->authManager;
		return $this->_permitted = $this->buildPermittedItems($this->_items, $auth);
	}


	/**
	 * @param array $items
	 * @param \yii\rbac\PhpManager $auth
	 * @return array
	 */
	protected function buildPermittedItems($items, $auth)
	{
		foreach ($items as $k => &$v) {
			$permission = (substr($v['url'][0], 0, 1) === '/' ? substr($v['url'][0], 1) : $v['url'][0]);
			if (!Yii::$app->user->can($permission)) {
				unset($v, $items[$k]);
				continue;
			}
			if (isset($v['items'])) {
				$v['items'] = $this->buildPermittedItems($v['items'], $auth);
			}
		}
		return $items;
	}


	/**
	 * @inherited
	 */
	public function offsetSet($offset, $value)
	{
		$this->_permitted = null;
		$this->_items[$offset] = $value;
	}

	/**
	 * @inherited
	 */
	public function offsetExists($var)
	{
		if (isset($this->items[$var])) {
			return true;
		}
		return false;
	}

	/**
	 * @inherited
	 */
	public function offsetUnset($var)
	{
		unset($this->_items[$var]);
	}

	/**
	 * @inherited
	 */
	public function offsetGet($var)
	{
		return $this->items[$var];
	}
}