<?php

namespace common\components;

use common\modules\certManager\components\ssl\Driver;
use common\modules\certManager\components\ssl\Exception as ExceptionSSL;
use Spyc;
use Yii;
use yii\base\Component;
use yii\base\Exception;
use yii\redis\Connection;

class Terminal extends Component
{
	const KEY_AUTOPROCESSING = 'terminal:autoProcessing';
	
	const ERROR_CONFIGURATION = 500;
	
	public $configPath;
	public $varsPath;
	
	public $vars;

	private $config;
	
	private $address;
	
	/* @var $connection Connection */
	private $redis;
	
	/**
	 * @inheritdoc
	 */
	public function init()
	{
		parent::init();
		
		$this->config = Spyc::YAMLLoad(Yii::getAlias($this->configPath));
		
		$this->redis = Yii::$app->redis;
		
	}
	
	
	public function getConfig()
	{
		return $this->config;
	}
	
	public function autoProcessingStart($params = [])
	{
		$certManager = Yii::$app->getModule('certManager');
		$privatePassword = !empty($params['privatePassword']) ? $params['privatePassword'] : null;
		
		// Проверка пароля приватного ключа
		try {
			$pKey = $certManager->getKeyData($this->config['autobot']['keyname'], 'private');
			$sslDriver = $certManager->ssl->getDriver();
			$sslDriver->openPrivateKey($pKey, $privatePassword);
		} catch (ExceptionSSL $ex) {
			switch ($ex->getCode()) {
				case Driver::ERROR_PKEY_INVALID :
					throw new Exception(Yii::t('app', 'Private key invalid password'));
					break;
				default:
					throw new Exception(Yii::t('app', 'SSL error'));
					break;
			}
		}
		
		$result = $this->redis->executeCommand('HMSET', [
			self::KEY_AUTOPROCESSING,
			'started', true,
			'startTime', time(),
			'privatePassword', $privatePassword, 
		]);
		
		if (!empty($this->config['mca']['dir']) && !empty($this->config['mca']['daemon']) && !empty($this->config['mca']['conf'])) {
			$command = $this->config['mca']['dir'] . "/" . $this->config['mca']['daemon'] ." -d -c " . $this->config['mca']['conf'];
			exec($command);
			
			syslog(LOG_INFO, "Start autoprocessing: " . $command);
		}
		
		$this->redis->executeCommand('HDEL', [self::KEY_AUTOPROCESSING, 'stopTime']);
		
		return $result;
	}
	
	public function autoProcessingStop()
	{
		$mcaConf = $this->getMcaConfig();		
		$pid = @file_get_contents($mcaConf['pid_file']);
		
		if (empty($pid)) {
			throw new Exception(Yii::t('app', 'Cannot read mca pid file data'), self::ERROR_CONFIGURATION);
		}
		
		$this->redis->executeCommand('HMSET', [self::KEY_AUTOPROCESSING,
			'started', false,
			'stopTime', time(),
			]);
		
		
		if (!empty($this->config['mca']['daemon'])) {
			$command = "kill  " . $pid;
			exec($command);
			
			syslog(LOG_INFO, "Stop autoprocessing: " . $command);
		}
		
		$this->redis->executeCommand('HDEL', [self::KEY_AUTOPROCESSING, 'privatePassword', 'startTime']);
		
		return true;
	}
	
	public function getMcaConfig()
	{
		$result = [];
		
		//$this->config['mca']['conf'] = '/var/www/cyberft.local/www/mca.conf';

		if (
			file_exists($this->config['mca']['conf'])
			&& is_readable($this->config['mca']['conf'])
		) {
			$conf = explode("\n", file_get_contents($this->config['mca']['conf']));
			foreach ($conf as $string) {
				$string = explode('=', trim($string));
				if (
					(2 != count($string)) 
					|| ('#' == $string[0][0])
				) {
					continue;
				}
				
				$result[trim($string[0])] = !empty($string[1]) ?  trim($string[1], ' \'":') : '';
			}
		}
		
		return $result;
	}
	
	public function setAutoProcessingConfig($key, $value = '')
	{
		if (!in_array($key, $this->getAutoProcessingFields())) {
			throw new Exception("Invalid autoprocessing key '{$key}'");
		}
		$this->redis->executeCommand('HSET', [self::KEY_AUTOPROCESSING, $key, $value]);
		return true;
	}
	
	public function getAutoProcessingConfig()
	{
		$result = [];
		$keys = $this->redis->executeCommand('HKEYS', [self::KEY_AUTOPROCESSING]);
		if ($keys) {
			$values = $this->redis->executeCommand('HMGET', array_merge([self::KEY_AUTOPROCESSING], $keys));
			$result = array_combine($keys, $values);
		}

		return $result;
	}
	
	public function getAutoProcessingFields()
	{
		return [
			'started', 'startTime', 'stopTime', 'privatePassword',
			'fixSender'
		];
	}
	
	/**
	 * 
	 * @return type
	 */
	public function getAddress()
	{
		return $this->config['address'];
	}
	
}
