<?php

namespace common\models;

use common\models\CertSignDocument as CertSignDocument;
use common\models\Cert as Cert;
use common\models\DocumentType as DocumentType;
use common\modules\docManager\models\SwtDocument;
use Yii;

/**
 * @property string $id
 * @property string $typeCode
 * @property integer $dateIn
 * @property integer $dateOut
 * @property integer $status
 * @property integer $statusInner
 * @property integer $errorId
 * @property string $absId
 * @property string $uuid
 * @property string $signAlgo
 * @property string $sender
 * @property string $rcpt
 * @property string $ref
 * @property string $path
 * @property string $pathSource
 *
 * @property CertSignDocument[] $certSignDocuments
 * @property Cert[] $certs
 * @property DocumentType $type
 */
class Document extends \yii\db\ActiveRecord {

	const STATUS_NOT_READY = 0;
	const STATUS_PICKUP = 1;
	const STATUS_SENT = 2;
	const STATUS_RECEIVED = 3;
	const STATUS_REJECTED = 4;
	const STATUS_DROP = 5;
	const STATUS_EXPORTED = 6;
	const STATUS_DELIVERED = 7;
	const STATUS_SIGNED = 8;
	const STATUS_SIGNED_INVALID = 9;
	
	const STATUS_INNER_ERROR = 0;
	const STATUS_INNER_SENDER_INVALID = 1;
	const STATUS_INNER_REGISTERED = 4;
	const STATUS_INNER_SIGNED = 5;

	const DEFAULT_ADDR = "*EMPTY*";
	
	//public $sender = self::DEFAULT_ADDRESS;
	//public $rcpt = self::DEFAULT_ADDRESS;
	
	
	/**
	 * File data
	 * @var type 
	 */
	public $data;
	
	/**
	 *
	 * @var SwtDocument 
	 */
	private $swtDocument;
	
	/**
	 *
	 * @var common\modules\docManager\Module
	 */
	protected $docManager;
	

	public function init() {
		parent::init();
		
		$this->docManager = Yii::$app->getModule('docManager');
	}
	
	/**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'document';
    }

	/**
	 * Все имеющиеся статусы документа
	 * @return array
	 */
	public static function getStatusValues()
	{
		return [
			self::STATUS_NOT_READY,
			self::STATUS_PICKUP,
			self::STATUS_SENT,
			self::STATUS_RECEIVED,
			self::STATUS_REJECTED,
			self::STATUS_DROP,
			self::STATUS_EXPORTED,
			self::STATUS_DELIVERED,
			self::STATUS_SIGNED,
			self::STATUS_SIGNED_INVALID,
		];
	} 

    /**
     * @inheritdoc
     */
    public function rules()
    {
		//parent::rules();
        return [
            [['status', 'statusInner', 'errorId'], 'integer'],
            [['uuid', 'pathSource'], 'required'],
            [['typeCode'], 'string', 'max' => 16],
            [['absId', 'uuid', 'signAlgo', 'sender', 'rcpt'], 'string', 'max' => 64],
            [['ref'], 'string', 'max' => 32],
            [['path'], 'string', 'max' => 255],
            [['sender', 'uuid'], 'unique', 'targetAttribute' => ['sender', 'uuid'], 'message' => 'The combination of UUID and Отправитель has already been taken.']
        ];
    }


	public function scenarios()
	{
		return array_merge(parent::scenarios(), [
			'register' => [
				'id', 'typeCode', 'dateIn', 'dateOut', 'status', 'statusInner',
				'errorId', 'absId', 'uuid', 'signAlgo', 'sender', 'rcpt', 'ref', 'path'
			],
			'view' => [
				'id', 'typeCode', 'dateIn', 'dateOut', 'status', 'statusInner',
				'errorId', 'absId', 'uuid', 'signAlgo', 'sender', 'rcpt', 'ref', 
				'path', 'pathSource'
			],
		]);
	}

	/**
	 * @inheritdoc
	 */
	public function attributeLabels()
	{
		return [
			'id' => Yii::t('app', 'ID'),
			'typeCode' => Yii::t('app', 'Тип'),
			'dateIn' => Yii::t('app', 'Зарегистрирован'),
			'dateOut' => Yii::t('app', 'Обработан'),
			'status' => Yii::t('app', 'Статус документа'),
			'statusInner' => Yii::t('app', 'Статус внутр документооборота'),
			'errorId' => Yii::t('app', 'Код ошибки'),
			'absId' => Yii::t('app', 'ABS ID'),
			'uuid' => Yii::t('app', 'UUID'),
			'signAlgo' => Yii::t('app', 'Подпись'),
			'sender' => Yii::t('app', 'Отправитель'),
			'rcpt' => Yii::t('app', 'Получатель'),
			'ref' => Yii::t('app', 'Процессинг ID'),
			'path' => Yii::t('app', 'Результирующий файл'),
			'pathSource' => Yii::t('app', 'Исходный файл'),
			'statusLabel' => Yii::t('app', 'Статус документа'),
			'statusInnerLabel' => Yii::t('app', 'Статус внутр'),
		];
	}

	public static function getUndefinedValue($attribute)
	{
		$values = [];

		return array_key_exists($attribute, $values) ? $values[$attribute] : 'undefined';
	}

	/**
	 * @return \yii\db\ActiveQuery
	 */
	public function getCertSignDocuments()
	{
		return $this->hasMany(CertSignDocument::className(), ['documentId' => 'id']);
	}

	/**
	 * @return \yii\db\ActiveQuery
	 */
	public function getCerts()
	{
		return $this->hasMany(Cert::className(), ['id' => 'certId'])->viaTable('{certSignDocument}', ['documentId' => 'id']);
	}

	public function setTypeCode($value) {
		
		/**
		 * @todo Убрать это нахер, разобраться с типизацией
		 */
		$this->typeCode = (string) $value;
		
	}
	
	/**
	 * @return \yii\db\ActiveQuery
	 */
	public function getTypeCode()
	{
		//return $this->hasOne(DocumentType::className(), ['code' => 'typeCode']);
		return $this->typeCode;
	}

	public function getStatusLabels()
	{
		return [
			self::STATUS_NOT_READY => 'Документ не готов к отправке',
			self::STATUS_PICKUP => 'Новое сообщение в очереди исходящих, готово к отправке',
			self::STATUS_SENT => 'Сообщение отправлено, ожидаем подтверждения',
			self::STATUS_RECEIVED => 'Cообщение получено процессингом',
			self::STATUS_REJECTED => 'Cообщение отвергнуто процессингом',
			self::STATUS_DROP => 'Новое сообщение в очереди входящих',
			self::STATUS_EXPORTED => 'Сообщение выгружено из очереди входящих',
			self::STATUS_DELIVERED => 'Cообщение получено получателем',
			self::STATUS_SIGNED => 'Cообщение подписано доверенными подписями',
			self::STATUS_SIGNED_INVALID => 'Сообщение содержит не валидную подпись',
		];
	}

	public function getStatusLabel()
	{
		return $this->getStatusLabels()[$this->status];
	}

	public function getStatusInnerLabels()
	{
		return [
			self::STATUS_INNER_ERROR => 'Ошибка исходного файла',
			self::STATUS_INNER_SENDER_INVALID => 'Сообщение содержит неверный адрес отправителя',
			self::STATUS_INNER_REGISTERED => 'Зарегистрирован',
			self::STATUS_INNER_SIGNED => 'Подписан',
		];
	}

	public function getStatusInnerLabel()
	{
		return $this->getStatusInnerLabels()[$this->statusInner];
	}

		
	public function getSwtDocument()
	{
		if (is_null($this->swtDocument)) {
			$this->swtDocument = new SwtDocument($this->pathSource);
		}
		
		return $this->swtDocument;
	}
	
	
	public function getResultFileContent()
	{
		$filepath = $this->docManager->getWorkingPath()  . '/' . $this->uuid;
		
		$result = false;
		if (file_exists($filepath)) {
			$result = file_get_contents($filepath);
		}
		
		return $result;
	}
	
	public function validate($attributeNames = null, $clearErrors = true)
	{
		parent::validate($attributeNames = null, $clearErrors = true);
		
		if (
				Yii::$app->terminal->address != $this->sender 
				&& !in_array($this->statusInner, [self::STATUS_INNER_ERROR])
		) {

			// Ошибку не выдавай, статус исправляй
			$this->statusInner = self::STATUS_INNER_SENDER_INVALID;
		}
		
		$swtDocument = $this->getSwtDocument();
		if (!$swtDocument->validate()) {
			$this->statusInner = self::STATUS_INNER_ERROR;
		}
		
		return !$this->hasErrors();
	}
}