<?php

namespace common\models;

use Yii;
use common\modules\certManager\Module as CertManagerModule;
use common\modules\certManager\validators\VerifySignValidator;
use \common\modules\certManager\components\ssl\Exception as CertManagerException;
use yii\db\Exception as DbException;

/**
 * @todo подумать над переносом в certManager
 */
class SignedRequest extends \common\base\Model {

    public $format;
    public $app;
    public $fingerprint;
    public $isDetached;
    public $raw;
    public $sign;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['format', 'app', 'fingerprint', 'isDetached', 'raw', 'sign'], 'required'],
            ['sign', 'validateSign']
        ];
    }


    public function validateSign($attribute, $params)
    {
        Yii::$app->getModule('certManager');
        try {
            if (
            !CertManagerModule::getInstance()->ssl
                ->verifyPkcs7(
                    $this->raw,
                    $this->$attribute,
                    CertManagerModule::getCertId($this->fingerprint),
                    $this->isDetached
                )
            ) {
                $this->addError($attribute, Yii::t('app', 'Подпись некорректна'));
			} else {
				return true;
			}
        } catch (CertManagerException $e) {
            if (CertManagerException::ERROR_INVALID_CERT_ID === $e->getCode()) {
                $this->addError($attribute, Yii::t('app', 'Не верный отпечаток сертификата'));
            } elseif (CertManagerException::ERROR_UNKNOWN_CERT_ID === $e->getCode()) {
				$this->addError($attribute, Yii::t('app', 'Запрашиваемый сертификат не зарегистрирован на Терминале'));
            } elseif (CertManagerException::ERROR_BROKEN_CERT_PATH === $e->getCode()) {
                $this->addError($attribute, Yii::t('app', 'Техническая ошибка. Ключ соответствующий сертификату не найден'));
            }
        } catch (DbException $e) {
            $this->addError($attribute, Yii::t('app', "Технические неполадки. Не удается подпись, попробуйте выполнить операцию позднее. \n{$e->getMessage()}"));
        } catch (\Exception $e) {
            $this->addError($attribute, Yii::t('app', "Не известная ошибка проверки подписи. \n{$e->getMessage()}"));
        }

        return false;
    }
}
