<?php

namespace common\models;

use common\modules\certManager\models\Cert;

use Yii;
use yii\base\NotSupportedException;
use yii\web\IdentityInterface;

/**
 * @property integer $id
 * @property string $name
 * @property string $email
 * @property string $authKey
 * @property string $activeCert
 */
class User extends \yii\db\ActiveRecord implements IdentityInterface
{

//	const STATUS_DELETED = 0;
//	const STATUS_ACTIVE = 10;
//	const ROLE_USER = 10;

	/**
	 * @inheritdoc
	 */
	public static function tableName()
	{
		return 'user';
	}

	public function getUserName()
	{
		return $this->name;
	}

	/**
	 * @inheritdoc
	 */
	public function rules()
	{
		return [
			[['email', 'name'], 'required'],
			[['email', 'name'], 'string', 'max' => 45]
		];
	}

	/**
	 * @inheritdoc
	 */
	public function attributeLabels()
	{
		return [
			'id' => Yii::t('app', 'ID'),
			'email' => Yii::t('app', 'Email'),
			'name' => Yii::t('app', 'Name'),
		];
	}

	/**
	 * @return \yii\db\ActiveQuery
	 */
	public function getCerts()
	{
		return $this->hasMany(Cert::className(), ['id' => 'certId'])->viaTable('userHasCert', ['userId' => 'id']);
	}

	/**
	 * @return \yii\db\ActiveQuery
	 */
	public function getRoles()
	{
		return $this
			->hasMany(Roles::className(), ['id' => 'rolesId'])->viaTable('userHasRole', ['userId' => 'id']);
	}


	/**
	 * @return null|Cert
	 */
	public function getActiveCert()
	{
		return $this
			->hasMany(Cert::className(), ['id' => 'certId'])
//			->where() @todo добавить условия по которым сертификат будет считаться гарантированно активным
			->viaTable('userHasCert', ['userId' => 'id'])
			->one();
	}

	/**
	 * @inheritdoc
	 */
	public static function findIdentity($id) {
		return static::findOne(['id' => $id]);
	}

	/**
	 * @inheritdoc
	 */
	public static function findIdentityByAccessToken($token, $type = null) {
		throw new NotSupportedException('"findIdentityByAccessToken" is not implemented.');
	}

	/**
	 * @inheritdoc
	 */
	public function getId() {
		return $this->getPrimaryKey();
	}

	/**
	 * @inheritdoc
	 */
	public function getAuthKey() {
		return $this->authKey;
	}

	/**
	 * @inheritdoc
	 */
	public function validateAuthKey($authKey) {
		return $this->getAuthKey() === $authKey;
	}

	/**
	 * Generates "remember me" authentication key
	 */
	public function generateAuthKey() {
		$this->authKey = Yii::$app->security->generateRandomString();
	}
}
