<?php

namespace common\models\base;

use Yii;

/**
 * This is the model class for table "documentType".
 *
 * @property integer $id
 * @property string $code
 * @property string $name
 * @property string $info
 *
 * @property Document[] $documents
 * @property UserHasDocumentType[] $userHasDocumentTypes
 * @property User[] $users
 */
class DocumentType extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'documentType';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['code'], 'required'],
            [['info'], 'string'],
            [['code'], 'string', 'max' => 16],
            [['name'], 'string', 'max' => 255],
            [['code'], 'unique']
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('app', 'ID'),
            'code' => Yii::t('app', 'Code'),
            'name' => Yii::t('app', 'Name'),
            'info' => Yii::t('app', 'Info'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDocuments()
    {
        return $this->hasMany(Document::className(), ['typeCode' => 'code']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserHasDocumentTypes()
    {
        return $this->hasMany(UserHasDocumentType::className(), ['documentTypeId' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUsers()
    {
        return $this->hasMany(User::className(), ['id' => 'userId'])->viaTable('{userHasDocumentType}', ['documentTypeId' => 'id']);
    }
}
