<?php

namespace common\models\form;

use common\modules\certManager\models\Cert;
use common\modules\certManager\Module as CertManager;
use common\models\User;
use Yii;
use \common\models\SignedRequest;

/**
 * Авторизация по электронной подписи
 */
class Login extends \common\base\Model
{

	public $identity = 'login';
	private $_user;
    /**
     * @var SignedRequest|array
     */
    public $signedRequest;
	
	/**
	 * @inheritdoc
	 */
	public function rules() {
		return [
            ['signedRequest', 'validateSignedRequest'],
            [['identity', 'signedRequest'], 'required'],
		];
	}


	public function validateSignedRequest($attribute, $params)
    {
        $signed = new SignedRequest();
        $signed->setAttributes($this->$attribute);
        if (!$signed->validate()) {
			$this->addError($attribute, $signed->getErrorsSummary(true));
            return false;
        }

        $this->signedRequest = $signed;
        return true;
    }
	
	
	/**
	 * @return boolean whether the user is logged in successfully
	 */
	public function login() {
		if ($this->validate() && ($user = $this->getUser())) {
			return Yii::$app->user->login($user);
		} else {
			return false;
		}
	}

	/**
	 * @return User|null
	 */
	public function getUser() {
        $certId = CertManager::getCertId($this->signedRequest->fingerprint);
        if (!$this->_user && $cert = Cert::findByCertId($certId)) {
			$this->_user = $cert->user;
		}
		return $this->_user;
	}

}
