<?php

namespace common\modules\docManager\components;

use Yii;
use yii\base\Component;
use yii\helpers\FileHelper;

class Storage extends Component {
	
	/**
	 * Допустимое кол-во файлов в папке
	 * 
	 * @var int 
	 */
	protected $maxFiles = 1000;
	protected $workingDir = '@docStorage/saved';


	public function init()
	{
		parent::init();

		if (!is_writable(Yii::getAlias($this->workingDir))) {
			throw new \yii\base\Exception('Cannot write into storage saved dir');
		}
		
	}

	
	/**
	 * Возвращает последнюю актуальную 
	 */
	public function getActualDir() 
	{
		$contents = $this->getDirSubfolders($this->workingDir);
		if (empty($contents)) {
			FileHelper::createDirectory(Yii::getAlias($this->workingDir . '/' . $this->generateDirectoryName()), 0775);
			$contents = $this->getDirSubfolders($this->workingDir);
		}
		
		$actual = end($contents);
		$actualContents = FileHelper::findFiles($actual);
		if (count($actualContents) >= $this->maxFiles) {
			$actual = Yii::getAlias($this->workingDir . '/' . $this->generateDirectoryName());
			FileHelper::createDirectory($actual);
		}
		
		return $actual;
	}
	
	protected function getDirSubfolders($dir)
	{
		$result = [];
		$path = Yii::getAlias($dir) . '/';
		$entries  = scandir($path);
		
		if (!empty($entries)) {
			foreach($entries as $entry) {
				if (
					('.' != $entry) 
					&& ('..' != $entry)
					&& (is_dir($path . $entry))
				) {
					$result[filectime($path . $entry)] = $path . $entry;
				}
			}
		}
		
		ksort($result); // сортировка по времени создания
		
		return $result;
	}
	
	
	protected function generateDirectoryName()
	{
		
		return uniqid();
	}
}
