<?php

namespace common\modules\docManager\models;

use Yii;
use yii\base\Model;
/**
 * Description of SwtDocument
 *
 * @author fuzz
 */

class SwtDocument extends Model 
{
	/* Attributes */
	public $file;
	
	protected $basicHeader;
	protected $applicationHeader;
	protected $userHeader;
	protected $trailer;
	
	private $sender;
	private $recipient;
	private $contentType;
	private $content;
	
	public function __construct($config = []) {
		
		if (is_string($config)) {
			$config = [
				'file'	=> $config
			];
		}
		
		parent::__construct($config);
 	}
	
	public function init()
	{
		parent::init();
		if ($this->file) {
			$this->loadFile($this->file);
		}
	}
		
	public function rules()
	{
		return [
			[
				['sender', 'recipient', 'contentType', 'content'],
				'required',
				'on'	=> ['default']
			],
			['basicHeader', 'string', 'length' => 25, 'on' => ['default']],
			['applicationHeader', 'string', 'min' => 17, 'on' => ['default']],
			[['sender', 'recipient'], 'string', 'length' => 12, 'on' => ['default']],
			[['contentType'], 'string', 'max' => 16, 'on' => ['default']],
		];
	}
	
	public function scenarios()
	{
		/**
		 * @todo Поддержать сценарии для разных типов документов
		 * В итоге тип будет определяться в зависимости от содержимого файла, 
		 * от типа будет зависеть набор аттрибутов и их содержимое 
		 */
		
		return array_merge(parent::scenarios(), [
			'default'	=> ['sender', 'recipient', 'contentType', 'content']
		]);
	}
	
	public function getData()
	{
		
		return $this->getAttributes($this->safeAttributes());
	}
	
	public function setContentType($value) 
	{
		$this->contentType = $value;
	}
	
	public function getContentType()
	{
		if (is_null($this->contentType)) {
			if (self::SCENARIO_DEFAULT == $this->scenario) {
				
				$this->contentType = substr($this->applicationHeader, 1, 3);
				if ($this->contentType) {
					$this->contentType = 'swift/' . $this->contentType;
				} else {
					$this->contentType = '';
				}
			}
		}
		
		return $this->contentType;
	}
	
	public function setContent($value)
	{
			
		$this->content = $value;
	}
	
	public function getContent()
	{
			
		return $this->content;
	}
	
	
	public function setSender($value)
	{
		$this->sender = $value;
	}

	public function getSender()
	{
		if (is_null($this->sender)) {
			if (self::SCENARIO_DEFAULT == $this->scenario) {
				if ($this->applicationHeader[0] == 'I') {

					$this->sender = substr($this->basicHeader, 3, 12);
				} else if ($this->applicationHeader[0] == 'O') {

					$this->sender = substr($this->applicationHeader, 14, 12);
				}
			}
		}
		return $this->sender;
	}
	
	public function setRecipient($value)
	{
		
		$this->recipient = $value;
	}
	
	public function getRecipient()
	{
		if (is_null($this->recipient)) {
			if (self::SCENARIO_DEFAULT == $this->scenario) {
				if ($this->applicationHeader[0] == 'I') {

					$this->recipient = substr($this->applicationHeader, 4, 12);
				} else if ($this->applicationHeader[0] == 'O') {

					$this->recipient = substr($this->basicHeader, 3, 12);
				}
			}
		}
		
		return $this->recipient;
	}
	
	public function getDirection()
	{
		return $this->applicationHeader[0];
	}
	
	/**
	 * 
	 * @param type $filePath
	 * @return boolean
	 */
	protected function loadFile($filePath)
	{
		$filePath = Yii::getAlias($filePath);
		if (!file_exists($filePath)) {
			return false;
		}
		
		$data = file_get_contents($filePath);
		
		$data = $this->parse($data);
		
		$this->file = $filePath;
		
		$this->basicHeader = empty($data[1]) ? null : $data[1];
		$this->applicationHeader = empty($data[2]) ? null : $data[2];
		$this->userHeader = empty($data[3]) ? null : $data[3];
		$this->content = empty($data[4]) ? null : $data[4];
		$this->trailer = empty($data[5]) ? null : $data[5];
		
		
		/**
		 * @todo Определить тип документа и установить сценарий
		 */
		
		
		return true;
	}


	protected function parse($input)
	{
		// разбиение пакета на сообщения по признаку начала собщения "{1:"
		$document = preg_replace('/(\{\s*1\s*\:)/', '%message%${1}', $input);
		$document = explode('%message%', $document);
		array_shift($document); // убрать пустой элемент
		
		// не обрабатываются файлы с более чем одним сообщением
		if (empty($document) || count($document) > 1) {
			return false;
		}
		
		$document = reset($document); 
		
		
		$fields = [];
		// разбиение сообщения на поля
		preg_match_all('/(\{([^{}]|(?R))*\})/s', $document, $matches);
		if (!empty($matches) && !empty($matches[1])) {
			$fields = $matches[1];
		}
		
		$result = [];
		// Формирование документа из полей сообщения
		foreach ($fields as $key => $field) {
			preg_match('/^\s*\{\s*([^\:]+)\:(.*)\}$/s', $field, $matches);
			$result[trim($matches[1])] = $matches[2];
		}
		
		return $result;
	}
	
	public function pack()
	{
		$basicHeader = $this->basicHeader;
		$applicationHeader  = $this->applicationHeader;
		$userHeader  = $this->userHeader;
		$content = $this->content;
		$trailer = $this->trailer;
		
		if (self::SCENARIO_DEFAULT == $this->scenario) {
			$contentType = str_replace('swift/', '', $this->getContentType());
			
			$applicationHeader = substr_replace($applicationHeader, $contentType, 1, 3);
			if ($this->applicationHeader[0] == 'I') {
				$basicHeader = substr_replace($basicHeader, $this->getSender(), 3, 12);
				$applicationHeader = substr_replace($applicationHeader, $this->getRecipient(), 4, 12);
			} else if ($this->applicationHeader[0] == 'O') {
				$basicHeader = substr_replace($basicHeader, $this->getRecipient(), 3, 12);
				$applicationHeader = substr_replace($applicationHeader, $this->getSender(), 14, 12);
			}
		}
		
		$result = [];
		if ($basicHeader) {
			$result[] = "{1:{$basicHeader}}";
		}
		
		if ($applicationHeader) {
			$result[] = "{2:{$applicationHeader}}";
		}
		
		if ($userHeader) {
			$result[] = "{3:{$userHeader}}";
		}
		
		if ($content) {
			$result[] = "{4:{$content}}";
		}

		if ($trailer) {
			$result[] = "{5:{$trailer}}";
		}
		
		return join('', $result);
	}
	
	/**
	 * Updates model source file
	 */
	public function save()
	{
		file_put_contents($this->file, $this->pack());
	}
}
