<?php
namespace console\controllers;

use Yii;
use yii\console\Controller;

/**
 * Role based auth
 * @package console\controllers
 */
class RbacController extends Controller
{

	protected $roles = ['admin', 'signer', 'inspector', 'autobot', 'nobody'];

	/**
	 * Help message
	 */
	public function actionIndex()
	{
		$this->run('/help', ['rbac']);
	}


	/**
	 * Init all rules
	 */
	public function actionInit()
	{
		$auth = Yii::$app->authManager;

		/**
		 * Simple actions
		 */
		$mainPage = $auth->createPermission('site/index');
		$mainPage->description = 'Main page';
		$auth->add($mainPage);

		$errorPage = $auth->createPermission('site/error');
		$errorPage->description = 'Error page';
		$auth->add($errorPage);

		$login = $auth->createPermission('site/login');
		$login->description = 'login';
		$auth->add($login);

		$logout = $auth->createPermission('site/logout');
		$logout->description = 'logout';
		$auth->add($logout);


		/**
		 * Entity permissions
		 */
		$documentList = $auth->createPermission('document/index');
		$documentList->description = 'List documents log';
		$auth->add($documentList);

		$documentView = $auth->createPermission('document/view');
		$documentView->description = 'View documents log item';
		$auth->add($documentView);
		
		$documentUpdate = $auth->createPermission('document/update');
		$documentUpdate->description = 'Update documents log item';
		$auth->add($documentUpdate);
		
		$documentInvalid = $auth->createPermission('document/invalid');
		$documentInvalid->description = 'Invalid documents log item';
		$auth->add($documentInvalid);
		
		$termIndex = $auth->createPermission('terminal/index');
		$termIndex->description = 'Terminal settings';
		$auth->add($termIndex);

		$autoProcessing = $auth->createPermission('terminal/auto-processing');
		$autoProcessing->description = 'Auto processing';
		$auth->add($autoProcessing);

		/**
		 * Roles
		 */
		$nobody = $auth->createRole('nobody');
		$auth->add($nobody);
		//$auth->addChild($nobody, $mainPage);
		$auth->addChild($nobody, $errorPage);
		$auth->addChild($nobody, $login);
		$auth->addChild($nobody, $logout);

		$admin = $auth->createRole('admin');
		$auth->add($admin);
		$auth->addChild($admin, $mainPage);
		$auth->addChild($admin, $documentList);
		$auth->addChild($admin, $documentView);
		$auth->addChild($admin, $documentUpdate);
		$auth->addChild($admin, $documentInvalid);
		$auth->addChild($admin, $termIndex);
		$auth->addChild($admin, $autoProcessing);

		/**
		 * @todo тестовый вариант, перенести соотв. логику когда будет реализовываться заведение пользователей
		 */
		$auth->assign($admin, 1);
		$auth->assign($admin, 2);
	}


	/**
	 * Сброс всех правил
	 */
	public function actionPurge()
	{
		$auth = Yii::$app->authManager;
		$auth->removeAll(); // Очищаем весь набор правил и ролей
	}


	/**
	 * Добавляем тестера со всеми возможными правами
	 */
	public function actionCreateTester()
	{
		if (YII_DEBUG) {
			$auth = Yii::$app->authManager;
			if ($auth->getRole('tester')) {
				print "Роль уже заведена\n";
				return Controller::EXIT_CODE_ERROR;
			}

			$tester = $auth->createRole('tester');
			$auth->add($tester);

			$permissions = $auth->getPermissions();
			foreach ($permissions as $v) {
				$auth->addChild($tester, $v);
			}
			print "Все добавили\n";
			return Controller::EXIT_CODE_NORMAL;
		}

		print "Данная роль доступна только в режиме отладки\n";
		return Controller::EXIT_CODE_ERROR;
	}

	/**
	 * Убиваем тестера
	 */
	public function actionRemoveTester()
	{
		$auth = Yii::$app->authManager;
		if ($auth->remove($auth->getRole('tester'))) {
			print "Роль тестера удалена\n";
			return Controller::EXIT_CODE_NORMAL;
		} else {
			return Controller::EXIT_CODE_ERROR;
		}

	}
	
	public function actionReload()
	{
		$this->actionPurge();
		$this->actionInit();
		$this->actionCreateTester();
		
	}
}