SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL,ALLOW_INVALID_DATES';

-- -----------------------------------------------------
-- Schema cyberswift
-- -----------------------------------------------------
CREATE TABLE `user` (
  `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
  `email` VARCHAR(45) NOT NULL,
  `name` VARCHAR(45) NOT NULL,
  PRIMARY KEY (`id`))
ENGINE = InnoDB;


CREATE TABLE `cert` (
  `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
  `validFrom`   TIMESTAMP NULL,
  `validBefore` TIMESTAMP NULL,
  `useBefore`   TIMESTAMP NULL,
  `terminalId` CHAR(12) NOT NULL COMMENT 'Идентификатор терминала где изначально зарегестрирован сертификат',
  `fingerprint` VARCHAR(64) NOT NULL COMMENT 'хэш сертификата',
  `status` VARCHAR(16) NULL COMMENT 'Дополнительный статус сертификата',
  `path` VARCHAR(255) NOT NULL,
  `country` CHAR(2) NULL COMMENT 'C = двухбуквенный код страны согласно ISO3166',
  `state` VARCHAR(64) NULL COMMENT 'ST = штат или область (край)',
  `locality` VARCHAR(64) NULL COMMENT 'L  = местоположение, обычно - город',
  `organization` VARCHAR(64) NULL COMMENT 'O  = организация - название компании',
  `organizationalUnit` VARCHAR(64) NULL COMMENT 'OU = подразделение организации, обычно - подразделение или наименование группы объектов',
  `commonName` VARCHAR(64) NULL COMMENT 'CN = общепринятое имя, обычно - наименование конечного субъекта, например www.example.com',
  `email` VARCHAR(64) NULL,
  `name` VARCHAR(64) NULL COMMENT 'GN = givenName',
  PRIMARY KEY (`id`),
  UNIQUE INDEX `id_unique` (`terminalId` ASC, `fingerprint` ASC))
ENGINE = InnoDB;


CREATE TABLE `role` (
  `id` SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT,
  `isSystem` TINYINT(1) NOT NULL DEFAULT 0,
  `code` VARCHAR(8) NULL COMMENT 'код роли в системе',
  `name` VARCHAR(64) NOT NULL,
  `info` TINYTEXT NULL,
  PRIMARY KEY (`id`))
ENGINE = InnoDB;


CREATE TABLE `actionLog` (
  `id` BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
  `userId` INT UNSIGNED NOT NULL,
  `time` INT UNSIGNED NULL,
  `app` VARCHAR(32) NULL,
  `controller` VARCHAR(32) NULL,
  `action` VARCHAR(32) NULL,
  `status` VARCHAR(32) NULL,
  `attributes` VARCHAR(1024) NULL COMMENT 'serialized params',
  `message` VARCHAR(1024) NULL,
  `info` TEXT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_systemLog_user1_idx` (`userId` ASC),
  INDEX `time` (`time` ASC),
  CONSTRAINT `fk_systemLog_user1`
    FOREIGN KEY (`userId`)
    REFERENCES `user` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


CREATE TABLE `document` (
  `id` BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
  `typeCode` VARCHAR(16) NULL COMMENT 'Тип документа',
  `dateIn` TIMESTAMP NULL COMMENT 'Зарегестрирован',
  `dateOut` TIMESTAMP NULL COMMENT 'Обработан',
  `status` TINYINT UNSIGNED NOT NULL DEFAULT 0 COMMENT 'Статус транспорта',
  `statusInner` SMALLINT UNSIGNED NOT NULL DEFAULT 0 COMMENT 'Статус внутр документооборота',
  `errorId` SMALLINT UNSIGNED NULL COMMENT 'Код ошибки',
  `absId` VARCHAR(64) NULL COMMENT 'ABS ID',
  `uuid` VARCHAR(64) NOT NULL COMMENT 'UUID',
  `signAlgo` VARCHAR(64) NULL COMMENT 'Подпись',
  `sender` VARCHAR(64) NOT NULL COMMENT 'Отправитель',
  `rcpt` VARCHAR(64) NULL COMMENT 'Получатель',
  `ref` VARCHAR(32) NULL COMMENT 'Процессинг ID',
  `path` VARCHAR(255) NOT NULL COMMENT 'Файл',
  PRIMARY KEY (`id`),
  UNIQUE INDEX `sender` (`sender` ASC, `uuid` ASC),
  INDEX `statusRcpt` (`status` ASC, `rcpt` ASC),
  INDEX `dateIn` (`dateIn` ASC),
  INDEX `typeCode` (`typeCode` ASC))
ENGINE = InnoDB;


CREATE TABLE `variable` (
  `name` VARCHAR(32) NOT NULL,
  `value` VARCHAR(1024) NULL,
  PRIMARY KEY (`name`))
ENGINE = InnoDB;


CREATE TABLE `userHasCert` (
  `userId` INT UNSIGNED NOT NULL,
  `certId` INT UNSIGNED NOT NULL,
  INDEX `fk_userHasCert_cert1_idx` (`certId` ASC),
  INDEX `fk_userHasCert_user1_idx` (`userId` ASC),
  PRIMARY KEY (`certId`),
  CONSTRAINT `fk_userHasCert_user1`
    FOREIGN KEY (`userId`)
    REFERENCES `user` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_userHasCert_cert1`
    FOREIGN KEY (`certId`)
    REFERENCES `cert` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


CREATE TABLE `documentType` (
  `id` SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT,
  `code` VARCHAR(16) NOT NULL,
  `name` VARCHAR(255) NULL,
  `info` TEXT NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `code_UNIQUE` (`code` ASC))
ENGINE = InnoDB;


CREATE TABLE `certSignDocument` (
  `certId` INT UNSIGNED NOT NULL,
  `documentId` BIGINT UNSIGNED NOT NULL,
  `datetime` INT NULL,
  PRIMARY KEY (`certId`, `documentId`),
  INDEX `fk_certHasDocument_document1_idx` (`documentId` ASC),
  INDEX `fk_certHasDocument_cert1_idx` (`certId` ASC),
  CONSTRAINT `fk_certHasDocument_cert1`
    FOREIGN KEY (`certId`)
    REFERENCES `cert` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_certHasDocument_document1`
    FOREIGN KEY (`documentId`)
    REFERENCES `document` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


CREATE TABLE `userHasDocumentType` (
  `userId` INT UNSIGNED NOT NULL,
  `documentTypeId` SMALLINT UNSIGNED NOT NULL,
  PRIMARY KEY (`userId`, `documentTypeId`),
  INDEX `fk_userHasDocumentType_documentType1_idx` (`documentTypeId` ASC),
  INDEX `fk_userHasDocumentType_user1_idx` (`userId` ASC),
  CONSTRAINT `fk_userHasDocumentType_user1`
    FOREIGN KEY (`userId`)
    REFERENCES `user` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_userHasDocumentType_documentType1`
    FOREIGN KEY (`documentTypeId`)
    REFERENCES `documentType` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


CREATE TABLE `userHasRole` (
  `userId` INT UNSIGNED NOT NULL,
  `roleId` SMALLINT UNSIGNED NOT NULL,
  PRIMARY KEY (`userId`, `roleId`),
  INDEX `fk_userHasRole_role1_idx` (`roleId` ASC),
  INDEX `fk_userHasRole_user1_idx` (`userId` ASC),
  CONSTRAINT `fk_userHasRole_user1`
    FOREIGN KEY (`userId`)
    REFERENCES `user` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_userHasRole_role1`
    FOREIGN KEY (`roleId`)
    REFERENCES `role` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


CREATE TABLE `contractor` (
  `id` INT NOT NULL,
  `name` VARCHAR(45) NULL,
  `info` TEXT NULL,
  PRIMARY KEY (`id`))
ENGINE = InnoDB;


SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;
