<?php
namespace frontend\controllers;

use Yii;
use common\models\LoginForm;
use frontend\models\PasswordResetRequestForm;
use frontend\models\ResetPasswordForm;
use frontend\models\SignupForm;
use frontend\models\ContactForm;
use yii\base\InvalidParamException;
use yii\web\BadRequestHttpException;
use yii\web\Controller;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;

/**
 * Site controller
 */
class SiteController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'only' => ['logout', 'signup'],
                'rules' => [
                    [
                        'actions' => ['signup'],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['logout'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'logout' => ['post'],
                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    public function actionIndex()
    {
        return $this->render('index');
    }
	
	public function actionTest()
    {
		

        $driver = Yii::$app->ssl->getDriver('OpenSSL');
		
		/*
		$keys = $driver->generateKeys('mypass');
		
		$keys = Yii::$app->ssl->storeFiles('mypass', $keys);
		var_dump($keys);
		exit;
		*/
		/*
		$csr = $driver->generateCertificateRequest([
			"countryName" => "UK",
			"stateOrProvinceName" => "Somerset",
			"localityName" => "Glastonbury",
			"organizationName" => "The Brain Room Limited",
			"organizationalUnitName" => "PHP Documentation Team",
			"commonName" => "Wez Furlong",
			"emailAddress" => "wez@example.com"
		], file_get_contents(Yii::getAlias('@keyStorage/mypass.key')), 'mypass');
		
		Yii::$app->ssl->storeData($csr, 'mypass', 'csr');
		*/
		
		/*
		$cert = $driver->signCertificateRequest(
				Yii::getAlias('@keyStorage/mypass.csr'), 
				null,
				Yii::getAlias('@keyStorage/mypass.key'), 
				'mypass'
				);
		Yii::$app->ssl->storeData($cert, 'mypass', 'cert');
		*/
		
//		$data = $driver->getCertificateData(Yii::getAlias('@keyStorage/mypass.cert'));
//		var_dump($data);
//		var_dump($driver->getCertificateFingerprint(Yii::getAlias('@keyStorage/mypass.cert')));
//		
		$testData = 'TEST DATA';
		
		$sign = $driver->sign($testData, Yii::getAlias('@keyStorage/mypass.key'), 'mypass');
		var_dump($driver->verify($testData, $sign, Yii::getAlias('@keyStorage/mypass.pub')));
		
		
		exit;
		
    }
	
	public function actionTestModule()
	{
		$certManager = Yii::$app->getModule('certManager');
		$certManager->verify('TEST DATA');
		
	}
	
	public function actionExample()
	{
		// Create the keypair
		$res=openssl_pkey_new();

		// Get private key
		openssl_pkey_export($res, $privkey, "PassPhrase number 1" );

		// Get public key
		$pubkey=openssl_pkey_get_details($res);
		$pubkey=$pubkey["key"];
		//var_dump($privkey);
		//var_dump($pubkey);

		// Create the keypair
		$res2=openssl_pkey_new();

		// Get private key
		openssl_pkey_export($res2, $privkey2, "This is a passPhrase *µà" );

		// Get public key
		$pubkey2=openssl_pkey_get_details($res2);
		$pubkey2=$pubkey2["key"];
		var_dump($privkey2);
		var_dump($pubkey2);

		$data = "Only I know the purple fox. Trala la !";

		openssl_seal($data, $sealed, $ekeys, array($pubkey, $pubkey2));

		var_dump("sealed");
		var_dump(base64_encode($sealed));
		var_dump(base64_encode($ekeys[0]));
		var_dump(base64_encode($ekeys[1]));
		
		var_dump($privkey2);
		// decrypt the data and store it in $open
		if (openssl_open($sealed, $open, $ekeys[1], openssl_pkey_get_private  ($privkey2  ,"This is a passPhrase *µà" ) ) ) {
			echo "here is the opened data: ", $open;
		} else {
			echo "failed to open data";
		}
	}
	
    public function actionLogin()
    {
        if (!\Yii::$app->user->isGuest) {
            return $this->goHome();
        }

        $model = new LoginForm();
        if ($model->load(Yii::$app->request->post()) && $model->login()) {
            return $this->goBack();
        } else {
            return $this->render('login', [
                'model' => $model,
            ]);
        }
    }

    public function actionLogout()
    {
        Yii::$app->user->logout();

        return $this->goHome();
    }

    public function actionContact()
    {
        $model = new ContactForm();
        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            if ($model->sendEmail(Yii::$app->params['adminEmail'])) {
                Yii::$app->session->setFlash('success', 'Thank you for contacting us. We will respond to you as soon as possible.');
            } else {
                Yii::$app->session->setFlash('error', 'There was an error sending email.');
            }

            return $this->refresh();
        } else {
            return $this->render('contact', [
                'model' => $model,
            ]);
        }
    }

    public function actionAbout()
    {
        return $this->render('about');
    }

    public function actionSignup()
    {
        $model = new SignupForm();
        if ($model->load(Yii::$app->request->post())) {
            if ($user = $model->signup()) {
                if (Yii::$app->getUser()->login($user)) {
                    return $this->goHome();
                }
            }
        }

        return $this->render('signup', [
            'model' => $model,
        ]);
    }

    public function actionRequestPasswordReset()
    {
        $model = new PasswordResetRequestForm();
        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            if ($model->sendEmail()) {
                Yii::$app->getSession()->setFlash('success', 'Check your email for further instructions.');

                return $this->goHome();
            } else {
                Yii::$app->getSession()->setFlash('error', 'Sorry, we are unable to reset password for email provided.');
            }
        }

        return $this->render('requestPasswordResetToken', [
            'model' => $model,
        ]);
    }

    public function actionResetPassword($token)
    {
        try {
            $model = new ResetPasswordForm($token);
        } catch (InvalidParamException $e) {
            throw new BadRequestHttpException($e->getMessage());
        }

        if ($model->load(Yii::$app->request->post()) && $model->validate() && $model->resetPassword()) {
            Yii::$app->getSession()->setFlash('success', 'New password was saved.');

            return $this->goHome();
        }

        return $this->render('resetPassword', [
            'model' => $model,
        ]);
    }
}
